// File: Calc.java
import java.io.*;
import antlr.CommonAST;
import antlr.collections.AST;
import antlr.RecognitionException;
import antlr.TokenStreamException;

class Calc {

	public static void main(String[] args) throws Exception {
		boolean b = true;

    try {
      if (args.length > 0) {
        String filename = args[0];
        if (processEquationFile(filename)) {
          return;
        }
      }

      float value = 0;
      BufferedReader reader = null;
      while (true) {
        System.out.print("Enter an expression: ");
        reader = new BufferedReader(new InputStreamReader(System.in));
        value = parseExpression(reader, "<stdin>");
        System.out.println("value is " + value);
      }
		}
		catch (TokenStreamException e) {
			System.err.println("exception: " + e);
		}
		catch (RecognitionException e) {
			System.err.println("exception: " + e);
		}
	}


  private static float parseExpression(Reader reader, String 
filename) {
    float r = 0;
    try {
      CalcLexer lexer = new CalcLexer(reader);
      lexer.setFilename(filename);
      CalcParser parser = new CalcParser(lexer);
      parser.setFilename(filename);
      parser.setASTNodeType("CalcAST");
      parser.setASTNodeClass("CalcAST");

      parser.expression(); // Parse the input expression
//      AST t = parser.getAST();
      CalcAST t = (CalcAST)parser.getAST();

      System.out.println("Tree in LIST notation: " + t.toStringTree
());
      CalcTreeWalker walker = new CalcTreeWalker();

      // Traverse the tree created by the parser
      r = walker.expression(t);

		} catch (TokenStreamException e) {
			System.err.println("exception: " + e);
		} catch (RecognitionException e) {
			System.err.println("exception: " + e);
		}

    return r;
  } // method parseExpression


  private static boolean processEquationFile(String filename) throws 
Exception {
    File f = new File(filename);
    if (f.exists() == false) {
      System.out.println("File: " + f.getAbsoluteFile() + " does not exist!");
      return false;
    }
    BufferedReader fileReader = new BufferedReader(new FileReader(f));
    StringReader reader = null;
    String line = null;
    float value = 0;
    while ((line = fileReader.readLine()) != null) {
      if (line.startsWith("//") || (line.length() == 0)) {
        continue;
      }
      System.out.println("\nProcessing equation: " + line);
      reader = new StringReader(line);
      value = parseExpression(reader, "<stdin>");
      System.out.println("value is " + value);
    }

    return true;
  } // method processEquationFile

} // class Calc
