// File: calc.g
class CalcParser extends Parser;
options {
  k = 4;                           // four token lookahead
  codeGenMakeSwitchThreshold = 2;  // Some optimizations
  codeGenBitsetTestThreshold = 3;
  defaultErrorHandler = false;     // Don't generate parser error handlers
  ASTLabelType = "CalcAST";
	buildAST = true;
}

tokens {
  UNARY_MINUS<AST=UnaryMinusNode>;
  PLUS<AST=PlusNode>;
  MINUS<AST=MinusNode>;
  STAR<AST=MultNode>;
  DIV<AST=DivNode>;
  INT<AST=IntNode>;
  CHAR<AST=CharNode>;
}


expression
  : logicalExpression SEMI!
  ;

logicalExpression
  : relExpression ((AND^ | OR^) relExpression)*
  ;

relExpression
  : simpleExpression ((LT^ | LE^ | EQ^ | GE^ | GT^ | NEQ^) 
simpleExpression)*
  ;

simpleExpression
:m:MINUS^  term      {#m.setType(UNARY_MINUS);}	 
	 {
       System.out.println("Got unary");
 //      #simpleExpression = #([UNARY_MINUS, "-"], #t);

//       #simpleExpression = antlr.ASTFactory.create(new UnaryMinusNode(LT(1))), #t);
//       #simpleExpression = new UnaryMinusNode(t);
     }
  | term ( (PLUS^ | MINUS^) term )*
  ;

term
  : factor ( (STAR^ | DIV^) factor )*
  ;

factor
  : INT
  | LPAREN! logicalExpression RPAREN!
  | CHAR;





class CalcLexer extends Lexer;
options {
  charVocabulary = '\0'..'\377';
  testLiterals = false;   // don't automatically test for literals
  k = 4;                  // four characters of lookahead
  caseSensitive = true;
  caseSensitiveLiterals = true;
}


WS
	:	(' '
	|	'\t'
	|	'\n'
	|	'\r')
		{ _ttype = Token.SKIP; }
	;

SEMI         options {paraphrase = ";";}     : ';'   ;
STAR         options {paraphrase = "*";}     : '*'   ;
PLUS         options {paraphrase = "+";}     : '+'   ;
MINUS        options {paraphrase = "-";}     : '-'   ;
DIV          options {paraphrase = "/";}     : '/'   ;
AND          options {paraphrase = "AND";}   : "AND" ;
OR           options {paraphrase = "OR";}    : "OR"  ;
EQ           options {paraphrase = "=";}     : '='   ;
NEQ          options {paraphrase = "<>";}    : "<>"  ;
LT           options {paraphrase = "<";}     : '<'   ;
LE           options {paraphrase = "<=";}    : "<="  ;
GE           options {paraphrase = ">=";}    : ">="  ;
GT           options {paraphrase = ">";}     : '>'   ;
LPAREN       options {paraphrase = "(";}     : '('   ;
RPAREN       options {paraphrase = ")";}     : ')'   ;


protected
DIGIT : '0'..'9';
INT   : (DIGIT)+;
CHAR  : 'a'..'z'; 		

