
 
import java.io.*;
import java.net.URL;
import java.util.*;

import java.util.regex.*;
 
import org.xml.sax.*;
import org.xml.sax.helpers.*;
 
 
/**
* A utility class that parses a Comma 
* Separated Values (CSV) file and outputs its 
* contents using SAX2 events. The format of CSV 
* that this class reads is identical to the export 
* format for Microsoft Excel. For simple values, the 
* CSV file may look like this:
* <pre>
* a,b,c
* d,e,f
* </pre>
* Quotes are used as delimiters when the values 
* contain commas:
* <pre>
* a,"b,c",d
* e,"f,g","h,i"
* </pre>
* And double quotes are used when the values 
* contain quotes. This parser is smart enough 
* to trim spaces around commas, as well.
*
* @author Eric M. Burke
*/
public class CSVXMLReader extends AbstractXMLReader {
 
  // an empty attribute for use with SAX
  private static final Attributes EMPTY_ATTR = new AttributesImpl( );
 
  /**
   * Parse a CSV file. SAX events are 
   * delivered to the chandler
   * that was registered via 
   * <code>setchandler</code>.
   *
   * @param input the comma separated 
   * values file to parse.
   */

   Stack stack = new Stack();

	public CSVXMLReader() {
	}


	public CSVXMLReader(ContentHandler chandler) {
		this.chandler = chandler;
	}

	 public void parse(InputSource input) throws IOException,
      SAXException {

		System.out.println(" in parse insrc ");

	 }

	 public void parse(String ruleName, String condition) throws IOException,
      SAXException {

	//	 	System.out.println(" in parse ");

    // if no handler is registered to receive events, don't bother
    // to parse the CSV file
    chandler = getContentHandler();
    if (chandler == null) {
      return;
    }
 
 /*
    // convert the InputSource into a BufferedReader
    BufferedReader br = null;
    if (input.getCharacterStream( ) != null) {
      br = new BufferedReader(input.getCharacterStream( ));
    } else if (input.getByteStream( ) != null) {
      br = new BufferedReader(new InputStreamReader(
          input.getByteStream( )));
    } else if (input.getSystemId( ) != null) {
      java.net.URL url = new URL(input.getSystemId( ));
      br = new BufferedReader(new InputStreamReader(url.openStream( )));
    } else {
      throw new SAXException("Invalid InputSource object");
    }
 */

	//	 	System.out.println(" in parse sdoc ");

//	setFeature("http://xml.org/sax/features/namespaces", true); 
//	setFeature("http://xml.org/sax/features/namespace-prefixes", true);


    chandler.startDocument();

	   AttributesImpl atts = new AttributesImpl();
       atts.addAttribute("", "rul", "xmlns:rul", "CDATA", "http://www.ruleml.org");

	chandler.startPrefixMapping("rul", "http://www.ruleml.org");

 
   // emit <csvFile>
    chandler.startElement("http://www.ruleml.org", "rules","rul:rules", atts);

	atts = new AttributesImpl();
    atts.addAttribute("", "name", "name", "CDATA", ruleName);



	chandler.startElement("http://www.ruleml.org","rule","rul:rule", atts);

	chandler.startElement("http://www.ruleml.org","condition","rul:condition",EMPTY_ATTR);
 
    // read each line of the file until EOF is reached
    String curLine = null;

	String value = "";
    BufferedReader reader = null;

	try
	{

		StringTokenizer nvptok = new StringTokenizer(condition, " ");
		while(nvptok.hasMoreTokens()) {
			String nvpair = nvptok.nextToken();

	//		 	System.out.println(" first token = " + nvpair);

		
			if(isOperator(nvpair) || stack.empty())  {

	//		 	System.out.println(" first isoper pushing = ");

				stack.push(nvpair);

			} else {

	//		 	System.out.println(" not oper ");

					String ch5 = (String) stack.peek();

					if(isOperator(ch5))  {
	//				 	System.out.println(" top elem oper. pushing... " + nvpair);
						stack.push(nvpair);
					}
					else	{
	//				 	System.out.println(" calling tallyho ... ");
						tallyHo(nvpair);
					}
			}

		}
		
	}
	catch (Exception e) {
		System.err.println("exception str: " + e);
	}


    // emit </csvFile>
    chandler.endElement("http://www.ruleml.org","condition","rul:condition");
	chandler.endElement("http://www.ruleml.org","rule","rul:rule");
	chandler.endElement("http://www.ruleml.org","rules","rul:rules");




	chandler.endPrefixMapping("rul");
    chandler.endDocument( );

  }

  private void tallyHo(String ch) throws Exception {

		String ch2 = (String) stack.pop();

	 //	System.out.println(" in tallyho popped ch " + ch2);


		if(!stack.empty()) {

		// 	System.out.println(" popping another ... ");

			String tag =  (String) stack.pop();
			String oper = getTagForOper(tag);


		// 	System.out.println(" got ... " + oper);

			chandler.startElement("http://www.ruleml.org", oper, "rul:"+oper, EMPTY_ATTR);
		
			chandler.startElement("http://www.ruleml.org","LeftParam","rul:LeftParam", EMPTY_ATTR);
			chandler.startElement("http://www.ruleml.org","variable", "rul:variable", EMPTY_ATTR);

			String ch2nq = stripQuotes(ch2);
			String chnq = stripQuotes(ch);


			chandler.characters(ch2nq.toCharArray(), 0, ch2nq.length( ));

			chandler.endElement("http://www.ruleml.org","variable","rul:variable");
			chandler.endElement("http://www.ruleml.org","LeftParam","rul:LeftParam");
			chandler.startElement("http://www.ruleml.org","RightParam", "rul:RightParam", EMPTY_ATTR);
			chandler.startElement("http://www.ruleml.org","variable", "rul:variable", EMPTY_ATTR);
			chandler.characters(chnq.toCharArray(), 0, chnq.length( ));

			chandler.endElement("http://www.ruleml.org","variable","rul:variable");
			chandler.endElement("http://www.ruleml.org","RightParam","rul:RightParam");
			chandler.endElement("http://www.ruleml.org", oper, "rul:"+oper);


	// 	System.out.println(" in tallyho aft emit ");


			if(!stack.empty()) {
				String ch3 = (String) stack.peek();


				if(!isOperator(ch2))  {
			//	 	System.out.println(" not oper. recursing... ");

					tallyHo("VAR");
				} else {
	//			 	System.out.println(" oper. pushing... VAR ");
					stack.push("VAR");
				}

			}
		}

}

	private String stripQuotes(String str) {

		 String patternStr = "\'";
         String replacementStr = "";

	   // Compile regular expression
          Pattern pattern = Pattern.compile(patternStr);
    
          // Replace all occurrences of pattern in input
          Matcher matcher = pattern.matcher(str);
          String output = matcher.replaceAll(replacementStr);

		return output;


	}

  private String getTagForOper(String oper) {

	  String operator = oper.trim();
  
	if(operator.equals("+")) 
		return "sum";
	else if(operator.equals("-")) 
		return "sub";
	else if(operator.equals("*")) 
		return "mul";
	else if(operator.equals("/")) 
		return "div";
	else if(operator.equals("&&")) 
		return "and";
	else if(operator.equals("||")) 
		return "or";
	else if(operator.equals("==")) 
		return "EQ";
	else if(operator.equals("<")) 
			return "LT";
	else if(operator.equals(">")) 
			return "GT";
	else if(operator.equals("<=")) 
			return "LTE";
	else if(operator.equals(">=")) 
			return "GTE";
	else if(operator.equals("!=")) 
			return "NEQ";


	else
		return "";
  
  }

	private boolean isOperator(String ch) {

		if(ch.equals("==") || ch.equals("!=") || 	ch.equals("<=") || 	ch.equals(">=") || 	
			ch.equals("+") || 	ch.equals("-") || ch.equals("*") ||	ch.equals("div") || 
			ch.equals("&&") || 	ch.equals("||") || 	ch.equals("<") || ch.equals(">") || 	
			ch.equals("^" ) ) 
					return true;
	
		return false;
	}

 
}

